import sys, signal, os

from helper_functions import parse_output_path
from inference.ReadTable import ReadTable
from inference.InferenceModel import InferenceModel
import tensorflow as tf
def main(args):
    tf.config.threading.set_intra_op_parallelism_threads(1)
    tf.config.threading.set_inter_op_parallelism_threads(1)
    mod = InferenceModel(args.model)  # Load model
    # mod_dict = load_model(args.model)
    # input_length = mod_dict[list(mod_dict)[0]].layers[0].input_shape[1]
    pos_reads_dir = parse_output_path(args.out_dir + 'pos_reads')

    # Load read table, start table manager
    read_table = ReadTable(args.fast5_in, args.out_dir + 'index_table.db', pos_reads_dir, mod.kmers, mod.input_length)
    read_manager_process = read_table.init_table()

    # ensure processes are ended after exit
    def graceful_shutdown(sig, frame):
        print("shutting down")
        read_manager_process.terminate()
        read_manager_process.join()
        sys.exit(0)

    # Differentiate between inference modes by defining when loop stops
    if args.inference_mode == 'watch':
        signal.signal(signal.SIGINT, graceful_shutdown)
        def end_condition():
            return True
    elif args.inference_mode == 'once':
        def end_condition():
            return len(os.listdir(args.fast5_in)) != 0
    else:
        raise ValueError(f'{args.inference_mode} is not a valid inference mode')

    # Start inference loop
    while end_condition():
        read_id, read, kmer = read_table.get_read_to_predict(unsplit=args.continuous_nn)
        if read is None: continue
        pred = mod.predict(read, kmer)
        # pred = np.any(mod_dict[kmer].predict(read) > 0.0)
        read_table.update_prediction(read_id, kmer, pred)
    else:
        read_manager_process.terminate()
        read_manager_process.join()
