from nns.keras_metrics_from_logits import precision, recall, binary_accuracy
import pandas as pd
import random
from pathlib import Path
import h5py
from inference.InferenceModel import InferenceModel
from db_building.TrainingRead import Read
import matplotlib.pyplot as plt
import numpy as np
import time
import tensorflow as tf
import tensorflow.keras.backend as K
from sklearn.metrics import PrecisionRecallDisplay

COMPLEMENT_DICT = {'A': 'T', 'T': 'A', 'C': 'G', 'G': 'C'}


def loss_fun(y_true, y_pred):  # just dummy to satisfy the stupid thing being there
    msk = np.zeros(100, dtype=bool)
    msk[50] = True
    y_pred_single = tf.boolean_mask(y_pred, msk, axis=1)
    return K.binary_crossentropy(K.cast(y_true, K.floatx()), y_pred_single,
                                 from_logits=True)

def precision_recall_at_threshold(threshold, target_model, target_name, read_path, ax):
    """

    :param threshold: Logit threshold to use
    :param target_model: Path to target model
    :param target_name: Name of target species in ground_truth csv
    :param ax: ax to plot on
    :return:
    """
    stride = 125
    num_of_reads_to_check = 100
    ground_truths_list = []
    frac_of_kmers_found_list = []
    mod = InferenceModel(target_model, threshold=threshold)
    print(f'{len(mod.kmers)=}')
    ground_truth_df = pd.read_csv(
        '/lustre/BIF/nobackup/noord087/HoiCarlos/16Sreads_mockcommunity/ground_truth_with_read_id_and_perc_id.csv')
    start = time.time()
    for counter, file in enumerate(
            Path(read_path).iterdir()):
        file_info = ground_truth_df[ground_truth_df['file name'] == file.name]
        read_is_target_species = target_name in str(
            file_info['species'])
        print(f"Target in read: {read_is_target_species}")
        # if read_is_target_species and random.random() > 0.9:
        #     # Reduce to 1/10 of what is normal
        #     continue
        with h5py.File(file, 'r') as f:
            raw_read_split_reads = Read(f, 'median').get_split_raw_read(250,
                                                                        stride)

        total_kmers_found = [kmer for kmer in mod.kmers if
                             mod.predict(raw_read_split_reads, kmer)]
        fraction_of_kmers_found = len(total_kmers_found) / len(mod.kmers)
        print(f'{fraction_of_kmers_found=}')
        print(counter / num_of_reads_to_check)
        ground_truths_list.append(int(read_is_target_species))
        frac_of_kmers_found_list.append(fraction_of_kmers_found)
        if counter > num_of_reads_to_check:
            break
    print(
        f'Total time for {num_of_reads_to_check} reads: {time.time() - start} seconds')
    PrecisionRecallDisplay.from_predictions(ground_truths_list,
                                            frac_of_kmers_found_list, name=f'{threshold=}', ax=ax)


def precision_recall_wrapper():
    # # Now doing it for rhodobacter
    # model_path = '/lustre/BIF/nobackup/noord087/mscthesis/baseless/baseless_250_hyperparam/r_sphaeroides_from16s_compiled.tar'
    # target_name = 'Rhodobacter sphaeroides'
    # model_path = '/lustre/BIF/nobackup/noord087/mscthesis/baseless/baseless_250_hyperparam/e_coli_from16s_compiled.tar'
    model_path = '/lustre/BIF/nobackup/noord087/mscthesis/benchmarking/output_16s_files_3_cores/baseless/escherichia_coli/compiled_model_fold0.tar'
    target_name = 'Escherichia coli'
    read_path = '/home/noord087/lustre_link/HoiCarlos/16Sreads_mockcommunity/demultiplexed_reads/files_for_initial_training/test'
    fig, ax = plt.subplots()
    for cutoff in np.arange(4, 6):
        precision_recall_at_threshold(cutoff, model_path, target_name, read_path, ax=ax)
    plt.show()


def main_plot_found_kmers_hist():
    stride = 125
    threshold = 4.5
    # Used for debugging
    num_of_reads_to_check = 1000
    kmers_in_neg = []
    kmers_in_pos = []
    model_path = '/lustre/BIF/nobackup/noord087/mscthesis/baseless/250_varied_negatives/e_coli_compiled_test.tar'
    mod = InferenceModel(model_path, threshold=threshold)

    ground_truth = pd.read_csv('/lustre/BIF/nobackup/noord087/HoiCarlos/16Sreads_mockcommunity/ground_truth_with_read_id_and_perc_id.csv')
    start = time.time()
    for counter, file in enumerate(Path('/home/noord087/lustre_link/HoiCarlos/16Sreads_mockcommunity/demultiplexed_reads/files_for_initial_training/test').iterdir()):
        file_info = ground_truth[ground_truth['file name'] == file.name]
        read_is_target_species = 'Escherichia coli' in str(file_info['species'])
        print(f"Target in read: {read_is_target_species}")
        with h5py.File(file, 'r') as f:
            # guppy_basecall = f['Analyses/Basecall_1D_000/BaseCalled_template/Fastq'][()].decode('ascii')
            # seq_guppy = guppy_basecall.splitlines()[1]
            raw_read_split_reads = Read(f, 'median').get_split_raw_read(250,
                                                                        stride)

        total_kmers_found = [kmer for kmer in mod.kmers if
                             mod.predict(raw_read_split_reads, kmer)]
        # print(f'{total_kmers_found=}')
        print(f'Total kmers: {len(total_kmers_found)}')
        print(counter/num_of_reads_to_check)
        print()
        if read_is_target_species:
            kmers_in_pos.append(len(total_kmers_found))
        else:
            kmers_in_neg.append(len(total_kmers_found))
        if counter > num_of_reads_to_check:
            break
    print(f'Total time for {num_of_reads_to_check} reads: {time.time() - start} seconds')
    bins = np.arange(0, 22, 1)
    plt.hist(kmers_in_neg, bins=bins, alpha=0.5, label='Neg')
    plt.hist(kmers_in_pos, bins=bins, alpha=0.5, label='Pos')
    plt.legend()
    plt.xlabel = 'Number of kmers found'
    plt.title(f'Logit threshold: {mod.threshold}')
    plt.show()


def main_scan_for_kmers_in_basecalls():
    # 24 kmers in E coli for which we trained models
    e_coli_kmers = ['AAACTTCC', 'ACGACTTC', 'ACTAGCGA', 'AGATGGAT', 'AGGAAGGG',
                    'AGTTTTAA', 'ATCCTCTC', 'CAAAGGAG', 'CAACACGA', 'CAAGCGGA',
                    'CCATTGTA', 'CCCGCAGA', 'CCGTACTC', 'CCTTCGCC', 'CTCTCAGA',
                    'CTTCATAC', 'CTTCCTCC', 'CTTCTTCC', 'CTTGCGGC', 'GCATCGAA',
                    'GCTTCCCA', 'GGAGTCGA', 'GGTATCTA', 'TAAGTCGA']
    # Get their reverse complements
    rev_com_e_coli_kmers = [
        ''.join([COMPLEMENT_DICT[n] for n in kmer][::-1]) for kmer in
        e_coli_kmers]

    ground_truth = pd.read_csv('/lustre/BIF/nobackup/noord087/HoiCarlos/16Sreads_mockcommunity/ground_truth_with_read_id_and_perc_id.csv')
    for file in Path('/home/noord087/lustre_link/HoiCarlos/16Sreads_mockcommunity/demultiplexed_reads/files_for_initial_training/test').iterdir():
        file_info = ground_truth[ground_truth['file name'] == file.name]
        print(f"Target in read: {'Escherichia coli' in str(file_info['species'])}")
        print(file.name)
        with h5py.File(file, 'r') as f:
            guppy_basecall = f['Analyses/Basecall_1D_000/BaseCalled_template/Fastq'][()].decode('ascii')
            seq_guppy = guppy_basecall.splitlines()[1]
            # tombo_events = f['Analyses/RawGenomeCorrected_000/BaseCalled_template/Events']['base'].astype(str)
            # seq_tombo = ''.join(tombo_events)

        kmers_found = sum(kmer in seq_guppy for kmer in e_coli_kmers)
        rev_kmers_found = sum(kmer in seq_guppy
                              for kmer in rev_com_e_coli_kmers)

        print(max(kmers_found, rev_kmers_found), 'of 24 found in target')
        print()


if __name__ == '__main__':
    precision_recall_wrapper()
